package carte;

import java.util.HashMap;

/**
 * Une <tt>HashMap</tt>, avec une valeur par dfaut qui est retourne par la
 * mthode <tt>get</tt>, au lieu de <tt>null</tt> quand il n'y a pas de valeur
 * associe.
 * 
 * @param <K>
 *          type des cls
 * @param <V>
 *          type des valeurs associes
 */
@SuppressWarnings("serial")
public class Etiquettes<K, V> extends HashMap<K, V> {
  private V _defaultValue;

  /**
   * Constructeur pour spcifier la valeur par dfaut.
   * 
   * @param defValue
   *          valeur par dfaut
   */
  public Etiquettes(V defValue) {
    this._defaultValue = defValue;
  }

  /**
   * @param k
   * @return la valeur associe  l'objet <tt>k</tt> (cette valeur peut tre
   *         <tt>null</tt>), ou la valeur par dfaut quand il n'y a pas de
   *         valeur associe.
   */
  @Override
  public V get(Object k) {
    return (containsKey(k) ? super.get(k) : _defaultValue);
  }
}
